// **************************************************************************
// optilogic.h
//		Version 1.5	2/01
//
//		See "OptiLogic Software Interface Definition" Manual for
//		definition of usage of these DLL calls
//
// Header File for the OptiLogic Protocol DLL.
//
// **************************************************************************
#ifdef __cplusplus
extern "C" {
#endif
#ifndef OPTILOGIC
#define OPTILOGIC
//
//	Protocol definitions
//
#define OL_IPX	1
#define OL_UDP	2
#define OPTISOCKET 0x983	// 2435 decimal
//
//	Define some useful macros 
//
typedef ULONG MODADDR;
typedef char* NETWORKADDR;
typedef short NETWORKPORT;
typedef USHORT NODEADDR;
//
//  Serial port definitions.
//
enum OL_SERIAL_PORTS {
	OL_BASE_SERIALPORT = 0x00,
	OL_SERIALPORT_1 = 0x01,
	OL_SERIALPORT_2,
	OL_SERIALPORT_3,
	OL_SERIALPORT_4
	};
enum OL_SERIAL_BAUD_RATES {
	OL_SERIAL_BAUD_300 = 0x1,OL_SERIAL_BAUD_1200, OL_SERIAL_BAUD_2400,
	OL_SERIAL_BAUD_4800, OL_SERIAL_BAUD_9600, OL_SERIAL_BAUD_19200,
	OL_SERIAL_BAUD_38400
	};
enum OL_SERIAL_DATA_BITS {
	OL_SERIAL_7_DATA_BITS = 0x7, 
	OL_SERIAL_8_DATA_BITS
	};
enum OL_SERIAL_PARITY {
	OL_SERIAL_NO_PARITY = 0x0,
	OL_SERIAL_ODD_PARITY,
	OL_SERIAL_EVEN_PARITY
	};
enum OL_SERIAL_STOP_BITS {
	OL_SERIAL_1_STOP_BIT = 0x1,
	OL_SERIAL_2_STOP_BITS,
	OL_SERIAL_15_STOP_BITS
	};
enum OL_SERIAL_CONTROL {
	OL_SERIAL_CONTROL_DISABLE,
	OL_SERIAL_CONTROL_ENABLE
};
enum OL34xx_BUTTON_FLAGS { 
	OL34xx_BUTTON_ALL=0x80,
	OL34xx_BUTTON_OR=0x40,
	OL34xx_BUTTON_AND=0x20 
	};
// ******************************************
//
//  Housekeeping Group (All of these functions 
//  are "Immediate Mode" functions)
//
// ******************************************
void	_stdcall OL_NetworkRetryCount(int nRetryCount);
void	_stdcall OL_NetworkTimeout(LONG timeout);
BOOL	_stdcall OL_IsNetworkInitialized(void);
ULONG	_stdcall OL_GetLastErrorCode(void);
ULONG	_stdcall OL_GetLastErrorLineNumber(void);
char*	_stdcall OL_GetLastErrorFile(void);
const char* _stdcall OL_GetLastErrorCodeString(void);
ULONG	_stdcall OL_GetNodeRetryCount(NODEADDR addr);
BOOL	_stdcall OL_ResetNodeRetryCount(NODEADDR addr);

// ******************************************
//
//	Session Group
//		Used to open & Close the Network
//
// ******************************************
BOOL	_stdcall OL_NetworkAddIPSearchAddress(char *strIPAddress);
BOOL	_stdcall OL_NetworkRemoveIPSearchAddress(char *strIPAddress);
BOOL	_stdcall OL_NetworkInit(NETWORKPORT port, BYTE protocol,int OL_Timeout, int MaxRetries);
void	_stdcall OL_NetworkClose(void);

// ******************************************
//
//	Node Group

//		Used to find out who's connected
//
// ******************************************
UINT	_stdcall OL_QueryNode(NODEADDR nodeaddress, BYTE *basetype, BYTE *moduletypes,
				BYTE *subtypes, BYTE *moduleVersions, BYTE *minversion,
				BYTE *majversion) ;
UINT	_stdcall OL_GetFirst(NODEADDR *nodeaddress, BYTE *basetype, BYTE moduletypes[],
				BYTE subtypes[], BYTE moduleVersions[], BYTE *minversion,
				BYTE *majversion);
UINT	_stdcall OL_GetNext(NODEADDR *nodeaddress, BYTE *basetype, BYTE moduletypes[],
				BYTE subtypes[], BYTE moduleVersions[], BYTE *minversion,
				BYTE *majversion);

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
//	IMMEDIATE MODE CALLS
//
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

// ******************************************
//
// Immediate Group I/O
//
// ******************************************


// Read digital inputs 
BOOL	_stdcall OL_ReadDigitalInput(NODEADDR Addr, USHORT modno, BYTE type, ULONG *inputdata);

// Write digital outputs
BOOL	_stdcall OL_WriteDigitalOutput(NODEADDR addr, USHORT modno, BYTE type, ULONG data);

// Read latched digital inputs 
BOOL	_stdcall OL_ReadLatchedDigitalInput(NODEADDR Addr, USHORT modno, BYTE type, ULONG *inputdata);

// Read digital outputs
BOOL	_stdcall OL_ReadDigitalOutput(NODEADDR addr, USHORT modno, BYTE type, ULONG *data);


// *********************************************************************************
// Purpose:
//	Set up output modules with a fail safe pattern in the event communication fail.
//
// Arguments:
//	addr		-	This is the address dialed into an RTU.
//	modno		-	The slot number the module is plugged into.
//	type		-	The type of module being used.
//						0x08 - OL2104 - 4 Digital outputs
//						0x09 - OL2108 - 8 Digital outputs
//						0x0A - OL2116 - 16 Digital outputs
//						0x0B - OL2124 - 24 Digital outputs
//						0x0C - OL2132 - 32 Digital outputs
//
//	fsType		- 	Fail safe type
//						1 - fail safe to all outputs off
//						2 - fail safe to the pattern fsPatern.
//						3 - fail safe to last state.
//
//	fsPattern	-	DWORD fail safe pattern (32-bits)
//
//	fsTime		-	Time to allow with no message from the host computer before
//					going to fail safe state.  In tenths of seconds.
//
// *********************************************************************************
BOOL	_stdcall OL_SetUpDigitalOutputFailSafe(NODEADDR addr, USHORT modno, 
											   BYTE byModuleType, BYTE fsType, 
											   DWORD fsPattern, BYTE fsTime);

// Read Counter/Digital Inputs 
BOOL	_stdcall OL_Read_Counter(NODEADDR Addr, USHORT modno, BYTE flags,
								 BYTE *countdata, ULONG *inData);
BOOL	_stdcall OL_Configure_Counter (NODEADDR Addr, USHORT modno, BYTE ctlflags,
									   BYTE dbSet1, BYTE dbSet2);
BOOL	_stdcall OL_Read_HS_Counter(NODEADDR Addr, USHORT modno, BYTE *countdata, 
									BYTE *freqdata, ULONG *inStat, ULONG *outStat);
BOOL	_stdcall OL_Configure_HS_Counter (NODEADDR Addr, USHORT modno, BYTE ctlflags, 
										  BYTE cfgflags, BYTE *preSet);

BOOL	_stdcall OL_HS_SendOutput_Range_Counter (NODEADDR Addr, USHORT modno, BYTE output, 
												 BYTE *minValue, BYTE *maxValue);

//	Read analog inputs
BOOL    _stdcall OL_ReadAnalogInput(NODEADDR addr, USHORT modno, BYTE type, USHORT *buffer);
BOOL	_stdcall OL_ReadAnalogInputs(NODEADDR addr, USHORT modno, USHORT *buffer,
									 BYTE type, BYTE numvalues);

//	Write analog outputs
BOOL	_stdcall OL_WriteAnalogOutput(NODEADDR addr, USHORT modno, BYTE type,
				USHORT *buffer);
BOOL	_stdcall OL_ConfigAnalogOutput(NODEADDR addr, USHORT modno, BYTE type, 
										BYTE range1, BYTE range2, BYTE range3, BYTE range4);

//	Serial ports calls
BOOL	_stdcall OL_ReadSerialData(NODEADDR addr, USHORT modno, BYTE type,
				enum OL_SERIAL_PORTS,BYTE *numbtes, BYTE *flags, BYTE *buffer);
BOOL	_stdcall OL_WriteSerialData(NODEADDR addr, USHORT modno, BYTE type,
				enum OL_SERIAL_PORTS, BYTE numbytes, BYTE *buffer, BYTE *left);
BOOL	_stdcall OL_ConfigureSerialPort(NODEADDR addr, USHORT modno, BYTE type,
				enum OL_SERIAL_PORTS, enum OL_SERIAL_BAUD_RATES,
				enum OL_SERIAL_DATA_BITS, enum OL_SERIAL_PARITY,
				enum OL_SERIAL_STOP_BITS);

// ******************************************
//
//	Immediate Mode Panel Group
//
// ******************************************

//	Operator Panel 3406
BOOL	_stdcall OL_OL3406_StatusControl(NODEADDR addr,  BYTE *data, BYTE *pbstatus);
BOOL	_stdcall OL_OL3406_ForceButtons (NODEADDR addr, enum OL34xx_BUTTON_FLAGS,
				BYTE ButtonData);
BOOL	_stdcall OL_OL3406_ConfigurePanel (NODEADDR addr, BYTE Conflags);
BOOL	_stdcall OL_OL3406_ReadPanelConfiguration (NODEADDR addr, BYTE *Conflags);

//	Operator Panel 3420
BOOL	_stdcall OL_OL3420_StatusRequest(NODEADDR addr, BYTE LEDState, BYTE *Status);
BOOL	_stdcall OL_OL3420_SendTextDisplayMessage(NODEADDR addr, BYTE line,
				char *message);
BOOL	_stdcall OL_OL3420_ConfigurePanel(NODEADDR addr, BYTE buttonstate);
BOOL	_stdcall OL_OL3420_ReadConfiguration(NODEADDR addr, BYTE *buttonstate);
BOOL	_stdcall OL_OL3420_ForceButtons (NODEADDR addr, enum OL34xx_BUTTON_FLAGS,
				BYTE ButtonData);

//	Operator Panel 3440
BOOL	_stdcall OL_OL3440_SendTextDisplayMessage(NODEADDR addr, BYTE line,
				char *message);

//	Operator Terminal 3850
BOOL	_stdcall OL_OL3850_StatusRequest(NODEADDR addr, BYTE LEDState, BYTE LEDFlash,
										BYTE *Status, double *ReturnDATA);
BOOL	_stdcall OL_OL3850_SendTextDisplayMessage(NODEADDR addr, BYTE line, char *message);
BOOL	_stdcall OL_OL3850_ConfigurePanel(NODEADDR addr, BYTE buttonstate);
BOOL	_stdcall OL_OL3850_ReadConfiguration(NODEADDR addr, BYTE *buttonstate);
BOOL	_stdcall OL_OL3850_ForceButtons(NODEADDR addr, enum OL34xx_BUTTON_FLAGS, BYTE ButtonData);
BOOL	_stdcall OL_OL3850_SendKeypadMessage(NODEADDR addr, BYTE line, char *message,
											 double StartDATA, BYTE decimalPoint);
BOOL	_stdcall OL_OL3850_SendArrowMessage(NODEADDR addr, BYTE line, char *message,
											unsigned long NumDATA, BYTE decimalPoint, unsigned long LowLimit,
											unsigned long HighLimit);

// SDS Master
BOOL	_stdcall OL_Send_Receive_SDS_Data (NODEADDR Addr, USHORT modno, 
										   BYTE outlength, USHORT *outdata, 
										   BYTE *inlength, USHORT *inData);

BOOL	_stdcall OL_Read_Status_Tables    (NODEADDR Addr, 
										   USHORT modno, 
										   USHORT *rtnLength, 
										   USHORT *rtnTable);

BOOL	_stdcall OL_Read_SDS_Device (NODEADDR Addr, USHORT modno, 
									 BYTE sendLength, USHORT *sendTable,
									 USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_SDS_Status_Diag (NODEADDR Addr, USHORT modno, 
									 BYTE sendLength, USHORT *sendTable,
									 USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_SDS_Service_Request (NODEADDR Addr, USHORT modno, 
										 BYTE sendLength, USHORT *sendTable,
										 USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_SDS_IO_Server_Config (NODEADDR Addr, USHORT modno, 
										  BYTE sendLength, USHORT *sendTable,
										  USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_SDS_Cancel_Special_Request (NODEADDR Addr, USHORT modno);
												
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//
//	BUFFERED MODE CALLS
//
// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

// ***********************************************
//
// Buffered Mode Housekeeping
//
// ***********************************************
void	_stdcall OL_Buffered_StartMessage();
BYTE	_stdcall OL_Buffered_ResetModule(NODEADDR addr, BYTE modno);
BOOL	_stdcall OL_Buffered_UpdateNodes(NODEADDR *nodearray, DWORD arraysize);
ULONG	_stdcall OL_Buffered_GetErrorCode(NODEADDR addr, BYTE modno);
ULONG	_stdcall OL_Buffered_GetErrorLineNumber(NODEADDR addr, BYTE modno);
char*	_stdcall OL_Buffered_GetErrorFile(NODEADDR addr, BYTE modno);
const char*	_stdcall OL_Buffered_GetErrorCodeString(NODEADDR addr, BYTE modno);

// ************************************************
//
//  Buffered Mode I/O Group
//
// ************************************************

// Read digital inputs 
BOOL	_stdcall OL_Buffered_ReadLatchedDigitalInput(NODEADDR Addr, USHORT modno, BYTE type, ULONG *inputdata);
BOOL	_stdcall OL_Buffered_ReadDigitalInput(NODEADDR Addr, USHORT modno, BYTE type,
				ULONG *inputdata);

// Read digital outputs
BOOL	_stdcall OL_Buffered_ReadDigitalOutput(NODEADDR Addr, USHORT modno, BYTE type, ULONG *inputdata);

// Write digital outputs
BOOL	_stdcall OL_Buffered_WriteDigitalOutput(NODEADDR addr, USHORT modno, BYTE type,
				ULONG data);

// Read Counter/Digital Inputs 
BOOL	_stdcall OL_Buffered_Read_Counter(NODEADDR Addr, USHORT modno, BYTE flags,
				BYTE *countdata, ULONG *inData);
BOOL	_stdcall OL_Buffered_Configure_Counter (NODEADDR Addr, USHORT modno, BYTE ctlflags,
				BYTE dbSet1, BYTE dbSet2);
BOOL	_stdcall OL_Buffered_Read_HS_Counter(NODEADDR Addr, USHORT modno, BYTE *countdata, 
											 BYTE *freqdata, BYTE *inStat, BYTE *outStat);
BOOL	_stdcall OL_Buffered_Configure_HS_Counter (NODEADDR Addr, USHORT modno, BYTE ctlflags, 
												   BYTE cfgflags, BYTE *preSet);

BOOL	_stdcall OL_Buffered_HS_SendOutput_Range_Counter (NODEADDR Addr, USHORT modno, BYTE output, 
														  BYTE *minValue, BYTE *maxValue);

//	Read analog inputs
BOOL	_stdcall OL_Buffered_ReadAnalogInputs(NODEADDR addr, USHORT modno,
				USHORT *buffer, BYTE type, BYTE numvalues);

//	Write analog outputs
BOOL	_stdcall OL_Buffered_WriteAnalogOutput(NODEADDR addr, USHORT modno, USHORT *buffer, 
											   BYTE type, BYTE numvalues);
BOOL	_stdcall OL_Buffered_ConfigAnalogOutput(NODEADDR addr, USHORT modno, BYTE type, 
												BYTE range1, BYTE range2, BYTE range3, 
												BYTE range4);

// 	Serial Port Calls
BOOL	_stdcall OL_Buffered_ReadSerialData(NODEADDR addr, USHORT modno, BYTE type,
											enum OL_SERIAL_PORTS port,BYTE *numbtes, 
											BYTE *flags, BYTE *buffer);
BOOL	_stdcall OL_Buffered_WriteSerialData(NODEADDR addr, USHORT modno, BYTE type,
											enum OL_SERIAL_PORTS port, BYTE numbytes,
											BYTE *buffer, BYTE *left);
BOOL	_stdcall OL_Buffered_ConfigureSerialPort(NODEADDR addr, USHORT modno, BYTE type, 
												enum OL_SERIAL_PORTS port,
												enum OL_SERIAL_BAUD_RATES baudrate, 
			  									enum OL_SERIAL_DATA_BITS databits,
												enum OL_SERIAL_PARITY parity, 
												enum OL_SERIAL_STOP_BITS stopbits);

// ************************************************
//
//	Buffered Mode Panel Group
//
// ************************************************

// Operator Panel 3406
BOOL	_stdcall OL_Buffered_OL3406_StatusControl(NODEADDR addr, BYTE *data, BYTE *pbstatus);
BOOL	_stdcall OL_Buffered_OL3406_ForceButtons (NODEADDR addr, enum OL34xx_BUTTON_FLAGS, BYTE ButtonData);
BOOL	_stdcall OL_Buffered_OL3406_ConfigurePanel (NODEADDR addr, BYTE ConFlags);
BOOL	_stdcall OL_Buffered_OL3406_ReadPanelConfiguration (NODEADDR addr, BYTE *ConFlags);

// 	Operator Panel 3420
BOOL	_stdcall OL_Buffered_OL3420_StatusRequest(NODEADDR addr, BYTE LEDState, BYTE *Status);
BOOL	_stdcall OL_Buffered_OL3420_SendTextDisplayMessage(NODEADDR addr, BYTE line, char *message);
BOOL	_stdcall OL_Buffered_OL3420_ConfigurePanel(NODEADDR addr, BYTE buttonstate);
BOOL	_stdcall OL_Buffered_OL3420_ReadConfiguration(NODEADDR addr, BYTE *buttonstate);
BOOL	_stdcall OL_Buffered_OL3420_ForceButtons (NODEADDR addr, enum OL34xx_BUTTON_FLAGS, BYTE ButtonData);


//	Operator Panel 3440
BOOL	_stdcall OL_Buffered_OL3440_SendTextDisplayMessage(NODEADDR addr, BYTE line,char *message);

//	Operator Terminal 3850
BOOL	_stdcall OL_Buffered_OL3850_StatusRequest(NODEADDR addr, BYTE LEDState,
												  BYTE LEDFlash, BYTE *Status, 
												  double *ReturnDATA);
BOOL	_stdcall OL_Buffered_OL3850_SendTextDisplayMessage(NODEADDR addr, BYTE line, char *message);
BOOL	_stdcall OL_Buffered_OL3850_ConfigurePanel(NODEADDR addr, BYTE buttonstate);
BOOL	_stdcall OL_Buffered_OL3850_ReadConfiguration(NODEADDR addr, BYTE *buttonstate);
BOOL	_stdcall OL_Buffered_OL3850_ForceButtons(NODEADDR addr, enum OL34xx_BUTTON_FLAGS, BYTE ButtonData);
BOOL	_stdcall OL_Buffered_OL3850_SendKeypadMessage(NODEADDR addr, BYTE line, char *message, double StartDATA,
													  BYTE decimalPoint);
BOOL	_stdcall OL_Buffered_OL3850_SendArrowMessage(NODEADDR addr, BYTE line,
													 char *message, unsigned long NumDATA,
													 BYTE decimalPoint,
													 unsigned long LowLimit, 
													 unsigned long HighLimit);
// 	SDS Master
BOOL	_stdcall OL_Buffered_Send_Receive_SDS_Data (NODEADDR Addr, USHORT modno, 
													BYTE outlength, USHORT *outdata, 
													BYTE *inlength, USHORT *inData);

BOOL	_stdcall OL_Buffered_Read_Status_Tables    (NODEADDR Addr, 
													USHORT modno, 
													USHORT *rtnLength, 
													USHORT *rtnTable);

BOOL	_stdcall OL_Buffered_Read_SDS_Device (NODEADDR Addr, USHORT modno, 
												BYTE sendLength, USHORT *sendTable,
												USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_Buffered_SDS_Status_Diag (NODEADDR Addr, USHORT modno, 
												BYTE sendLength, USHORT *sendTable,
												USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_Buffered_SDS_Service_Request (NODEADDR Addr, USHORT modno, 
													BYTE sendLength, USHORT *sendTable,
													USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_Buffered_SDS_IO_Server_Config (NODEADDR Addr, USHORT modno, 
													BYTE sendLength, USHORT *sendTable,
													USHORT *rtnLength, USHORT *rtnTable);

BOOL	_stdcall OL_Buffered_SDS_Cancel_Special_Request (NODEADDR Addr, USHORT modno);

#endif // end of #ifndef OPTILOGIC
#ifdef __cplusplus
}
#endif
